import React, { useState } from 'react'
import { useDrop } from 'react-dnd'
import update from 'immutability-helper'
import { is, fromJS } from 'immutable'
import { Tabs } from 'antd'
import Utils from '@/utils/utils.js'
import Card from './card'
import './index.scss'

const { TabPane } = Tabs

const Container = ({list, handleList, handleMenu, deleteMenu, doubleClickCard }) => {
  const [cards, setCards] = useState(list)
  const moveCard = (id, atIndex) => {
    const { card, index } = findCard(id)
    if (!card) return
    
    const _cards = update(cards, { $splice: [[index, 1], [atIndex, 0, card]] })

    handleList(_cards)
  }

  if (!is(fromJS(cards), fromJS(list))) {
    setCards(list)
  }

  const findCard = id => {
    const card = cards.filter(c => `${c.uuid}` === id)[0]
    return {
      card,
      index: cards.indexOf(card),
    }
  }

  const [, drop] = useDrop({
    accept: 'tabs',
    drop(item) {
      if (item.hasOwnProperty('originalIndex')) {
        return
      }

      let newcard = {}
      
      newcard.uuid = Utils.getuuid()
      newcard.label = 'tab'
      newcard.icon = ''
      newcard.type = item.subType
      newcard.linkTab = ''
      newcard.supMenu = 'mainTable'
      newcard.focus = true

      let targetId = ''

      if (item.dropTargetId) {
        targetId = item.dropTargetId
        delete item.dropTargetId
      } else if (cards.length > 0) {
        targetId = cards[cards.length - 1].uuid
      }

      const { index: overIndex } = findCard(`${targetId}`)
      const _cards = update(cards, { $splice: [[overIndex + 1, 0, newcard]] })

      handleList(_cards, newcard)
    }
  })

  const doubleClickBab = id => {
    const { card } = findCard(id)
    doubleClickCard(card)
  }

  const editCard = id => {
    const { card } = findCard(id)
    handleMenu(card)
  }

  const delCard = id => {
    const { card } = findCard(id)
    deleteMenu(card)
  }

  return (
    <div ref={drop} className="ant-row model-tab-drag-list">
      <Tabs defaultActiveKey="0">
        {cards.map((card, index) => (
          <TabPane tab={
            <Card
              key={card.uuid}
              id={`${card.uuid}`}
              card={card}
              moveCard={moveCard}
              findCard={findCard}
              editCard={editCard}
              delCard={delCard}
              doubleClickCard={doubleClickBab}
            />
          } key={`${index}`}>
            《{card.label}》标签
          </TabPane>
        ))}
      </Tabs>
      {cards.length === 0 ?
        <div className="commontab-drawarea-placeholder">
          请添加标签
        </div> : null
      }
    </div>
  )
}
export default Container
